/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
using System;
using NAnt.Core.Types;
using NAnt.Core;
using NAnt.Core.Attributes;
using TopCoder.Test.Framework.Servers;

namespace TopCoder.Test.Framework
{

    /// <summary>
    /// <p>
    /// This class is derived from the Element class.
    /// This class represents "server" XML element to be used in the NAnt build file.
    /// This element should be used inside the "serverapptest" NAnt task.
    /// That task is represented by the ServerAppTestTask class.
    /// </p>
    /// <p>
    /// The attributes supported by this element are:<br/>
    /// 1) "class" - the name of class representing the server, should be a valid name of the
    /// class derived from the Server abstract class. This attribute is required.<br/>
    /// 2) "startserver"- specifies if the server should
    /// be started by the Component when running tests.
    /// This attribute is not required, by default is true.<br/>
    /// 3) "initserver" - specifies if the server should
    /// be intialized with data by the Component when running tests.
    /// This attribute is not required, by default is true.<br/>
    /// </p>
    /// <p>
    /// The nested XML elements supported are:<br/>
    /// "option" - can be used zero or more times, represents a particular option of the server.<br/>
    /// "credential" - can be used zero or one times, represents a credentials used to login to the server.<br/>
    /// "sourcefiles" - can be used zero or more times, represents a files used to initialize the server with data<br/>
    /// </p>
    /// <p>
    /// This class is mutable, i.e. not thread-safe.
    /// </p>
    /// </summary>
    ///
    /// <author>real_vg</author>
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    [ElementName("server")]
    public class ServerElement : Element
    {

        /// <summary>
        /// <p>
        /// This variable represents the "class" attribute.
        /// "class" attribute represents the name of class derived from Server,
        /// which represents the server described by this element.
        /// This attribute is required.
        /// Initial value is null.
        /// It can be accessed via "ClassName" property.
        /// </p>
        /// </summary>
        private string className = null;

        /// <summary>
        /// <p>
        /// This variable represents the server described by this element
        /// Initially is null.
        /// </p>
        /// </summary>
        private Server server = null;

        /// <summary>
        /// <p>
        /// This variable represents the "startserver" attribute.
        /// "startserver" attribute specifies if the server should
        /// be started by the Component when running tests.
        /// This attribute is not required, the default value is true.
        /// Initially is set to true.
        /// Can be accessed via the StartServer property.
        /// </p>
        /// </summary>
        private bool startServer = true;

        /// <summary>
        /// <p>
        /// This variable represents the "initserver" attribute.
        /// "initserver" attribute specifies if the server should
        /// be intialized with data by the Component when running tests.
        /// This attribute is not required, the default value is true.
        /// Initially is set to true.
        /// Can be accessed via the InitServer property.
        /// </p>
        /// </summary>
        private bool initServer = true;

        /// <summary>
        /// This variable represents the collection of nested "option" XML elements, which
        /// represent the options of the server.
        /// The options can be any key-value pairs, where keys and values are strings.
        /// The specified by the "class" attribute, class derived from Server
        /// may interpret the options in a freevolous way.
        /// This nested element can possibly be ommited if the specified server
        /// doesn't require it.
        ///
        /// Can be accessed via the "Options" property.
        /// </summary>
        private OptionCollection options = new OptionCollection();

        /// <summary>
        /// This variable represents the nested "credentials" XML element, which
        /// represents the credentials used to authenticate to the server.
        /// This includes user name, password an domain.
        /// This nested element can possibly be ommited,
        /// it depends on the particular server.
        ///
        /// Can be accessed via the "Credentials" property.
        /// </summary>
        private Credential credentials = new Credential();

        /// <summary>
        /// This variable represents the nested "sourcefiles" XML element, which
        /// represents the files used when initializing the server with data.
        /// This can be an application to deploy for an application server or
        /// some SQL scripts, DB tables, etc. for a data server,  or some other files
        /// the particular server may need to be initialized.
        /// This nested element can possibly be ommited if the
        /// specified by "class" attribute class derived from Server
        /// permits it to be ommited.
        ///
        /// Can be accessed via the "sourceFiles" property.
        /// </summary>
        private FileSet sourceFiles = new FileSet();

        /// <summary>
        /// This property provides the access to the "className" variable.
        /// The "className" variable represents the "class" attribute.
        /// "class" attribute represents the name of class derived from Server,
        /// which represents the server described by this element.
        /// This attribute is required.
        /// </summary>
        ///
        /// <value>The name of class derived from Server, which represents the server described by
        /// this element.</value>
        [TaskAttribute("class", Required=true)]
        public string ClassName
        {
            get
            {
                return className;
            }
            set
            {
                className = value;
            }
        }

        /// <summary>
        /// This property provides the access to the "startServer" variable.
        /// The "startServer" variable represents "startserver" attribute.
        /// "startserver" attribute specifies if the server should
        /// be started by the Component when running tests.
        /// This attribute is not required, the default value is true.
        /// This attribute is required.
        /// </summary>
        ///
        /// <value>The value which indicates if the server should be started by the Component when
        /// running tests.</value>
        [TaskAttribute("startserver")]
        public bool StartServer
        {
            get
            {
                return startServer;
            }
            set
            {
                startServer = value;
            }
        }


        /// <summary>
        /// This property provides the access to the "initServer" variable.
        /// The "initServer" variable represents "initserver" attribute.
        /// "initserver" attribute specifies if the server should
        /// be intialized with data by the Component when running tests.
        /// This attribute is not required, the default value is true.
        /// </summary>
        ///
        /// <value>The value which indicates if the server should be intialized with data by the
        /// Component when running tests.</value>
        [TaskAttribute("initserver")]
        public bool InitServer
        {
            get
            {
                return initServer;
            }
            set
            {
                initServer = value;
            }
        }


        /// <summary>
        /// This property provides the access to the "options" variable.
        /// The "options" variable represents the collection of nested "option" XML elements which
        /// represent the options of the server.
        /// The options can be any key-value pairs, where keys and values are strings.
        /// The specified by the "class" attribute, class derived from Server
        /// may interpret the options in a freevolous way.
        /// This nested element can possibly be ommited if the specified server
        /// doesn't require it.
        /// </summary>
        ///
        /// <value>The collection of nested "option" XML elements which
        /// represent the options of the server.</value>
        [BuildElementCollection("options", "option")]
        public OptionCollection Options
        {
            get
            {
                return options;
            }
        }


        /// <summary>
        /// This property provides the access to the "credentials" variable.
        /// The "credentials" variable represents the nested "credentials" XML element, which
        /// represents the credentials used to authenticate to the server.
        /// This includes user name, password an domain.
        /// This nested element can possibly be ommited,
        /// it depends on the particular server.
        /// </summary>
        ///
        /// <value>The nested "credentials" XML element, which
        /// represents the credentials used to authenticate to the server.</value>
        [BuildElement("credentials")]
        public Credential Credentials
        {
            get
            {
                return credentials;
            }
            set
            {
                credentials = value;
            }
        }


        /// <summary>
        /// This property provides the access to the "sourceFiles" variable.
        /// The "sourceFiles" variable represents the nested "sourcefiles" XML element, which
        /// represents the files used when initializing the server with data.
        /// This can be an application to deploy for an application server or
        /// some SQL scripts, DB tables, etc. for a data server,  or some other files
        /// the particular server may need to be initialized.
        /// This nested element can possibly be ommited if the
        /// specified by "class" attribute class derived from Server
        /// permits it to be ommited.
        /// </summary>
        ///
        /// <value>The nested "sourcefiles" XML element, which
        /// represents the files used when initializing the server with data.</value>
        [BuildElement("sourcefiles")]
        public FileSet SourceFiles
        {
            get
            {
                return sourceFiles;
            }
            set
            {
                sourceFiles = value;
            }
        }

        /// <summary>
        /// <p>
        /// Creates an instance of the class.
        /// </p>
        /// </summary>
        public ServerElement()
        {
        }

        /// <summary>
        /// Returns an instance of class derived from Server class,
        /// which represents the server describe by this XML element.
        /// The name of class to create instance of is taken from the ClassName property.
        /// </summary>
        ///
        /// <returns>An instance of class derived from Server class, which represents the server describe
        /// by this XML element.</returns>
        /// <exception cref="ServerException">If any error happens when creating an instance of Server.</exception>
        public Server GetServerInstance()
        {
            // Check if the instance of class derived from Server
            // was not created before, if it was, return the stored instance
            if (server != null)
            {
                return server;
            }

            try
            {
                // Get Type instance, representing the server class
                Type type = Type.GetType(ClassName);

                // Create an instance of the class
                server = (Server) Activator.CreateInstance(type);
            }
            catch (Exception e)
            {
                throw new ServerException("Failed to create the server instance.", e);
            }

            // Copy the property values
            server.StartServer = startServer;
            server.InitServer = initServer;
            server.Options = options;
            server.Credentials = credentials;
            server.SourceFiles = sourceFiles;
            server.Project = Project;

            return this.server;

        }
    }
}
