/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
using System;
using NUnit.Framework;
using NAnt.Core.Types;

namespace TopCoder.Test.Framework.Servers
{

    /// <summary>
    /// Unit test class for <c>ApplicationServer</c> class.
    /// </summary>
    ///
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    [TestFixture]
    public class ApplicationServerTest
    {

        /// <summary>
        /// Private <c>MockApplicationServer</c> variable used for test.
        /// </summary>
        private MockApplicationServer server;

        /// <summary>
        /// Sets up the test environment. A valid <c>MockApplicationServer</c> object is created.
        /// </summary>
        [SetUp]
        protected void SetUp()
        {
            server = new MockApplicationServer();
        }

        /// <summary>
        /// Tests accuracy of constructor <c>ApplicationServer()</c> of <c>ApplicationServer</c> class.
        /// The fields should be initialized properly.
        /// </summary>
        [Test]
        public void TestApplicationServerAccuracy()
        {
            Assert.IsTrue(server.StartServer, "StartServer should be set as true.");
            Assert.IsTrue(server.InitServer, "InitServer should be set as true.");
            Assert.IsNull(server.Options, "Options should be set as null.");
            Assert.IsNull(server.Credentials, "Credentials should be set as null.");
            Assert.IsNull(server.SourceFiles, "SourceFiles should be set as null.");
        }

        /// <summary>
        /// Tests inheritance of constructor <c>ApplicationServer()</c> of <c>ApplicationServer</c> class.
        /// <c>ApplicationServer</c> class should be inherited from <c>Server</c> class.
        /// </summary>
        [Test]
        public void TestApplicationServerInheritance()
        {
            Assert.IsTrue(((object)server) is Server,
                "ApplicationServer class should be inherited from Server class.");
        }

        /// <summary>
        /// Tests accuracy of constant <c>DEPLOYMENT_DIR</c> of <c>ApplicationServer</c> class.
        /// The fields should be initialized properly.
        /// </summary>
        [Test]
        public void TestConstantDeploymentDirAccuracy()
        {
            Assert.AreEqual("deployment_dir", ApplicationServer.DEPLOYMENT_DIR,
                "DEPLOYMENT_DIR constant should be correct.");
        }



        /// <summary>
        /// Tests accuracy of <c>DeploymentDirectory</c> getter property of <c>ApplicationServer</c> class.
        /// The field value should be gotten correctly.
        /// </summary>
        [Test]
        public void TestDeploymentDirectoryGetterAccuracyNull()
        {
            OptionCollection options = new OptionCollection();
            options.Add(TestFrameworkTestHelper.CreateOption(ApplicationServer.DEPLOYMENT_DIR, null));
            server.Options = options;

            Assert.IsNull(server.DeploymentDirectory, "DeploymentDirectory should be null.");
        }

        /// <summary>
        /// Tests accuracy of <c>DeploymentDirectory</c> getter property of <c>ApplicationServer</c> class.
        /// The field value should be gotten correctly.
        /// </summary>
        [Test]
        public void TestDeploymentDirectoryGetterAccuracyNonNull()
        {
            // set options
            OptionCollection options = new OptionCollection();
            options.Add(TestFrameworkTestHelper.CreateOption(ApplicationServer.DEPLOYMENT_DIR, "sample dir"));
            server.Options = options;

            Assert.AreEqual("sample dir", server.DeploymentDirectory,
                "DeploymentDirectory should be correct.");
        }

        /// <summary>
        /// Tests failure of <c>DeploymentDirectory</c> getter property of <c>ApplicationServer</c> class.
        /// Option specified is not set will lead to failure.
        /// Expect <c>ServerException</c>.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestDeploymentDirectoryGetterFailureNoSpecifiedOption()
        {
            // set options
            OptionCollection options = new OptionCollection();
            options.Add(TestFrameworkTestHelper.CreateOption("wrong dir", "sample dir"));
            server.Options = options;

            string temp = server.DeploymentDirectory;
        }

        /// <summary>
        /// Tests failure of <c>DeploymentDirectory</c> getter property of <c>ApplicationServer</c> class.
        /// Options not set will lead to failure.
        /// Expect <c>ServerException</c>.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestDeploymentDirectoryGetterFailureNullOptions()
        {
            // set options
            server.Options = null;

            string temp = server.DeploymentDirectory;
        }


    }

}
