/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
using System;
using NUnit.Framework;
using NAnt.Core.Types;

namespace TopCoder.Test.Framework.Servers
{

    /// <summary>
    /// Unit test class for <c>DatabaseServer</c> class.
    /// </summary>
    ///
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    [TestFixture]
    public class DatabaseServerTest
    {

        /// <summary>
        /// Private <c>MockDatabaseServer</c> variable used for test.
        /// </summary>
        private MockDatabaseServer server;

        /// <summary>
        /// Sets up the test environment. A valid <c>MockDatabaseServer</c> object is created.
        /// </summary>
        [SetUp]
        protected void SetUp()
        {
            server = new MockDatabaseServer();
        }

        /// <summary>
        /// Tests accuracy of constructor <c>DatabaseServer()</c> of <c>DatabaseServer</c> class.
        /// The fields should be initialized properly.
        /// </summary>
        [Test]
        public void TestDatabaseServerAccuracy()
        {
            Assert.IsTrue(server.StartServer, "StartServer should be set as true.");
            Assert.IsTrue(server.InitServer, "InitServer should be set as true.");
            Assert.IsNull(server.Options, "Options should be set as null.");
            Assert.IsNull(server.Credentials, "Credentials should be set as null.");
            Assert.IsNull(server.SourceFiles, "SourceFiles should be set as null.");
        }

        /// <summary>
        /// Tests inheritance of constructor <c>DatabaseServer()</c> of <c>DatabaseServer</c> class.
        /// <c>DatabaseServer</c> class should be inherited from <c>Server</c> class.
        /// </summary>
        [Test]
        public void TestDatabaseServerInheritance()
        {
            Assert.IsTrue(((object)server) is Server,
                "DatabaseServer class should be inherited from Server class.");
        }

        /// <summary>
        /// Tests accuracy of constant <c>DB_DRIVER</c> of <c>DatabaseServer</c> class.
        /// The fields should be initialized properly.
        /// </summary>
        [Test]
        public void TestConstantDbDriverAccuracy()
        {
            Assert.AreEqual("db_driver", DatabaseServer.DB_DRIVER,
                "DB_DRIVER constant should be correct.");
        }

        /// <summary>
        /// Tests accuracy of constant <c>DB_CONNECTION_STR</c> of <c>DatabaseServer</c> class.
        /// The fields should be initialized properly.
        /// </summary>
        [Test]
        public void TestConstantDbConnectionStrAccuracy()
        {
            Assert.AreEqual("db_connection_str", DatabaseServer.DB_CONNECTION_STR,
                "DB_CONNECTION_STR constant should be correct.");
        }

        /// <summary>
        /// Tests accuracy of constant <c>SQL_INIT_SCRIPT</c> of <c>DatabaseServer</c> class.
        /// The fields should be initialized properly.
        /// </summary>
        [Test]
        public void TestConstantSqlInitScriptAccuracy()
        {
            Assert.AreEqual("sql_init_script", DatabaseServer.SQL_INIT_SCRIPT,
                "SQL_INIT_SCRIPT constant should be correct.");
        }

        /// <summary>
        /// Tests accuracy of constant <c>SQL_CLEANUP_SCRIPT</c> of <c>DatabaseServer</c> class.
        /// The fields should be initialized properly.
        /// </summary>
        [Test]
        public void TestConstantSqlCleanupScriptAccuracy()
        {
            Assert.AreEqual("sql_cleanup_script", DatabaseServer.SQL_CLEANUP_SCRIPT,
                "SQL_CLEANUP_SCRIPT constant should be correct.");
        }


        /// <summary>
        /// Tests accuracy of <c>DatabaseDriver</c> getter property of <c>DatabaseServer</c> class.
        /// The field value should be gotten correctly.
        /// </summary>
        [Test]
        public void TestDatabaseDriverGetterAccuracyNull()
        {
            OptionCollection options = new OptionCollection();
            options.Add(TestFrameworkTestHelper.CreateOption(DatabaseServer.DB_DRIVER, null));
            server.Options = options;

            Assert.IsNull(server.DatabaseDriver, "DatabaseDriver should be null.");
        }

        /// <summary>
        /// Tests accuracy of <c>DatabaseDriver</c> getter property of <c>DatabaseServer</c> class.
        /// The field value should be gotten correctly.
        /// </summary>
        [Test]
        public void TestDatabaseDriverGetterAccuracyNonNull()
        {
            // set options
            OptionCollection options = new OptionCollection();
            options.Add(TestFrameworkTestHelper.CreateOption(DatabaseServer.DB_DRIVER, "sample dir"));
            server.Options = options;

            Assert.AreEqual("sample dir", server.DatabaseDriver,
                "DatabaseDriver should be correct.");
        }

        /// <summary>
        /// Tests failure of <c>DatabaseDriver</c> getter property of <c>DatabaseServer</c> class.
        /// Option specified is not set will lead to failure.
        /// Expect <c>ServerException</c>.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestDatabaseDriverGetterFailureNoSpecifiedOption()
        {
            // set options
            OptionCollection options = new OptionCollection();
            options.Add(TestFrameworkTestHelper.CreateOption("wrong dir", "sample dir"));
            server.Options = options;

            string temp = server.DatabaseDriver;
        }

        /// <summary>
        /// Tests failure of <c>DatabaseDriver</c> getter property of <c>DatabaseServer</c> class.
        /// Options not set will lead to failure.
        /// Expect <c>ServerException</c>.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestDatabaseDriverGetterFailureNullOptions()
        {
            // set options
            server.Options = null;

            string temp = server.DatabaseDriver;
        }

        /// <summary>
        /// Tests accuracy of <c>DatabaseConnectionString</c> getter property of <c>DatabaseServer</c> class.
        /// The field value should be gotten correctly.
        /// </summary>
        [Test]
        public void TestDatabaseConnectionStringGetterAccuracyNull()
        {
            OptionCollection options = new OptionCollection();
            options.Add(TestFrameworkTestHelper.CreateOption(DatabaseServer.DB_CONNECTION_STR, null));
            server.Options = options;

            Assert.IsNull(server.DatabaseConnectionString, "DatabaseConnectionString should be null.");
        }

        /// <summary>
        /// Tests accuracy of <c>DatabaseConnectionString</c> getter property of <c>DatabaseServer</c> class.
        /// The field value should be gotten correctly.
        /// </summary>
        [Test]
        public void TestDatabaseConnectionStringGetterAccuracyNonNull()
        {
            // set options
            OptionCollection options = new OptionCollection();
            options.Add(TestFrameworkTestHelper.CreateOption(DatabaseServer.DB_CONNECTION_STR, "sample dir"));
            server.Options = options;

            Assert.AreEqual("sample dir", server.DatabaseConnectionString,
                "DatabaseConnectionString should be correct.");
        }

        /// <summary>
        /// Tests failure of <c>DatabaseConnectionString</c> getter property of <c>DatabaseServer</c> class.
        /// Option specified is not set will lead to failure.
        /// Expect <c>ServerException</c>.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestDatabaseConnectionStringGetterFailureNoSpecifiedOption()
        {
            // set options
            OptionCollection options = new OptionCollection();
            options.Add(TestFrameworkTestHelper.CreateOption("wrong dir", "sample dir"));
            server.Options = options;

            string temp = server.DatabaseConnectionString;
        }

        /// <summary>
        /// Tests failure of <c>DatabaseConnectionString</c> getter property of <c>DatabaseServer</c> class.
        /// Options not set will lead to failure.
        /// Expect <c>ServerException</c>.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestDatabaseConnectionStringGetterFailureNullOptions()
        {
            // set options
            server.Options = null;

            string temp = server.DatabaseConnectionString;
        }



        /// <summary>
        /// Tests accuracy of <c>SqlInitScript</c> getter property of <c>DatabaseServer</c> class.
        /// The field value should be gotten correctly.
        /// </summary>
        [Test]
        public void TestSqlInitScriptGetterAccuracyNull()
        {
            OptionCollection options = new OptionCollection();
            options.Add(TestFrameworkTestHelper.CreateOption(DatabaseServer.SQL_INIT_SCRIPT, null));
            server.Options = options;

            Assert.IsNull(server.SqlInitScript, "SqlInitScript should be null.");
        }

        /// <summary>
        /// Tests accuracy of <c>SqlInitScript</c> getter property of <c>DatabaseServer</c> class.
        /// The field value should be gotten correctly.
        /// </summary>
        [Test]
        public void TestSqlInitScriptGetterAccuracyNonNull()
        {
            // set options
            OptionCollection options = new OptionCollection();
            options.Add(TestFrameworkTestHelper.CreateOption(DatabaseServer.SQL_INIT_SCRIPT, "sample dir"));
            server.Options = options;

            Assert.AreEqual("sample dir", server.SqlInitScript,
                "SqlInitScript should be correct.");
        }

        /// <summary>
        /// Tests failure of <c>SqlInitScript</c> getter property of <c>DatabaseServer</c> class.
        /// Option specified is not set will lead to failure.
        /// Expect <c>ServerException</c>.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestSqlInitScriptGetterFailureNoSpecifiedOption()
        {
            // set options
            OptionCollection options = new OptionCollection();
            options.Add(TestFrameworkTestHelper.CreateOption("wrong dir", "sample dir"));
            server.Options = options;

            string temp = server.SqlInitScript;
        }

        /// <summary>
        /// Tests failure of <c>SqlInitScript</c> getter property of <c>DatabaseServer</c> class.
        /// Options not set will lead to failure.
        /// Expect <c>ServerException</c>.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestSqlInitScriptGetterFailureNullOptions()
        {
            // set options
            server.Options = null;

            string temp = server.SqlInitScript;
        }


        /// <summary>
        /// Tests accuracy of <c>SqlCleanUpScript</c> getter property of <c>DatabaseServer</c> class.
        /// The field value should be gotten correctly.
        /// </summary>
        [Test]
        public void TestSqlCleanUpScriptGetterAccuracyNull()
        {
            OptionCollection options = new OptionCollection();
            options.Add(TestFrameworkTestHelper.CreateOption(DatabaseServer.SQL_CLEANUP_SCRIPT, null));
            server.Options = options;

            Assert.IsNull(server.SqlCleanUpScript, "SqlCleanUpScript should be null.");
        }

        /// <summary>
        /// Tests accuracy of <c>SqlCleanUpScript</c> getter property of <c>DatabaseServer</c> class.
        /// The field value should be gotten correctly.
        /// </summary>
        [Test]
        public void TestSqlCleanUpScriptGetterAccuracyNonNull()
        {
            // set options
            OptionCollection options = new OptionCollection();
            options.Add(TestFrameworkTestHelper.CreateOption(DatabaseServer.SQL_CLEANUP_SCRIPT, "sample dir"));
            server.Options = options;

            Assert.AreEqual("sample dir", server.SqlCleanUpScript,
                "SqlCleanUpScript should be correct.");
        }

        /// <summary>
        /// Tests failure of <c>SqlCleanUpScript</c> getter property of <c>DatabaseServer</c> class.
        /// Option specified is not set will lead to failure.
        /// Expect <c>ServerException</c>.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestSqlCleanUpScriptGetterFailureNoSpecifiedOption()
        {
            // set options
            OptionCollection options = new OptionCollection();
            options.Add(TestFrameworkTestHelper.CreateOption("wrong dir", "sample dir"));
            server.Options = options;

            string temp = server.SqlCleanUpScript;
        }

        /// <summary>
        /// Tests failure of <c>SqlCleanUpScript</c> getter property of <c>DatabaseServer</c> class.
        /// Options not set will lead to failure.
        /// Expect <c>ServerException</c>.
        /// </summary>
        [Test, ExpectedException(typeof(ServerException))]
        public void TestSqlCleanUpScriptGetterFailureNullOptions()
        {
            // set options
            server.Options = null;

            string temp = server.SqlCleanUpScript;
        }

    }

}
